#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup()
{
	ofBackground(0);

	// Load shader.
	auto normalSettings = ofShader::TransformFeedbackSettings();
	normalSettings.shaderFiles[GL_VERTEX_SHADER] = "shaders/normal.vert";
	normalSettings.varyingsToCapture = { "vPosition", "vColor" };
	normalShader.setup(normalSettings);

	// Load mesh.
	sphereMesh = ofMesh::icosphere(100.0, 4);

	// Allocate buffer (vec4 for position, vec4 for color).
	sphereBuffer.allocate(sizeof(glm::vec4) * 2 * sphereMesh.getNumIndices(), GL_STATIC_DRAW);

	// Setup parameters.
	resolution.set("resolution", 1.0, 0.0, 2.0);
	displacement.set("displacement", 10.0, 0.0, 100.0);

	// Setup GUI.
	guiPanel.setup("Feedback", "settings.json");
	guiPanel.add(resolution);
	guiPanel.add(displacement);
}

//--------------------------------------------------------------
void ofApp::update()
{
	rotationAngle += 10.0 * ofGetLastFrameTime();

	// Bind the buffer and draw the mesh, which will end up stored in the buffer.
	normalShader.beginTransformFeedback(GL_TRIANGLES, sphereBuffer);
	{
		normalShader.setUniform1f("uResolution", resolution);
		normalShader.setUniform1f("uDisplacement", displacement);
		sphereMesh.draw();
	}
	normalShader.endTransformFeedback(sphereBuffer);

	// Set the buffer as positions and colors in the VBO.
	// Stride is the size of 2 * vec4.
	// color is offset by the size of the position vec4.
	sphereVbo.setVertexBuffer(sphereBuffer, 4, sizeof(glm::vec4) * 2, 0);
	sphereVbo.setColorBuffer(sphereBuffer, sizeof(glm::vec4) * 2, sizeof(glm::vec4));
}

//--------------------------------------------------------------
void ofApp::draw()
{
	camera.begin();
	ofEnableDepthTest();
	{
		ofRotateXDeg(rotationAngle);
		ofRotateYDeg(rotationAngle);

		sphereVbo.draw(GL_TRIANGLES, 0, sphereVbo.getNumVertices());
	}
	ofDisableDepthTest();
	camera.end();

	// Draw the gui.
	ofSetColor(255);
	guiPanel.draw();
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key)
{
	if (key == ' ')
	{
		sphereBuffer.bind(GL_ARRAY_BUFFER);
		glm::vec4 * verts = sphereBuffer.map<glm::vec4>(GL_READ_ONLY);
		ofMesh saveMesh;
		for (int i = 0; i < sphereVbo.getNumVertices(); ++i)
		{
			saveMesh.addVertex(verts[i * 2 + 0]);
			saveMesh.addColor(ofFloatColor(verts[i * 2 + 1].r, verts[i * 2 + 1].g, verts[i * 2 + 1].b, verts[i * 2 + 1].a));
		}
		saveMesh.save(ofGetTimestampString("%Y%m%d-%H%M%S") + ".ply");
		sphereBuffer.unmap();
	}
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}
